from helferklassen import KnotenSymbol, KantenSymbol, Lesen

# Beschreibt einen Knoten
class Knoten:

    # Besetzt die Attribute und legt das Knotensymbol an.
    def __init__(self, bezeichner, x, y):
        self.bezeichner = bezeichner
        self.symbol = KnotenSymbol(x, y, 20, "weiss", bezeichner)
        teile = bezeichner.split("|")
        self.gefaess8 = int(teile[0])
        self.gefaess5 = int(teile[1])
        self.gefaess3 = int(teile[2])

    # Meldet den Bezeichner des Knotens zurück.
    def BezeichnerGeben(self):
        return self.bezeichner

    # Meldet das Symbol des Knotens zurück.
    def SymbolGeben(self):
        return self.symbol

    # Setzt die Farbe der Darstellung
    def FarbeSetzen(self, f):
        self.symbol.FarbeSetzen(f)
    
    # Meldet zurück, ob an diesem Knoten eines der Gefäße die gewünschte Füllmenge hat
    def IstFuellmengeVorhanden(self, fuellmenge):
        # Entsprechend der Teilaufgabe a ergänzen
        return False


class GraphMatrix:

    # Baut die Datenstruktur auf
    def __init__(self):
        self.knoten = []
        self.kanten = []
        self.matrix = []
        self.warteliste = []
        self.fertigeKnoten = []

    # Einfügen eines neuen Knoten in den Graphen
    def KnotenEinfuegen(self, bezeichner, x, y):
        self.knoten.append(Knoten(bezeichner, x, y))
        for zeile in self.matrix:
            zeile.append(-1)
        self.matrix.append([])
        for zeile in self.matrix:
            self.matrix[-1].append(-1)

    # Gibt den Knoten mit dem angegebenen Bezeichner zurück.
    def KnotenGeben(self, bezeichner):
        for k in self.knoten:
            if k.BezeichnerGeben() == bezeichner:
                return k
        return None

    # Gibt die interne Nummer des Knotens
    def KnotenNummerGeben(self, bezeichner):
        for i in range(len(self.knoten)):
            if self.knoten[i].BezeichnerGeben() == bezeichner:
                return i
        return -1

    # Gibt die Bezeichnung eines Knotens mit der internen Knotennummer
    def KnotenBezeichnerGeben(self, KnotenNummer):
        if knotenNummer < len(self.knoten) and knotenNummer >= 0:
            return self.knoten[knotenNummer].BezeichnerGeben()
        else:
            return ""

    # Einfügen einer Kante in den Graphen
    def KanteEinfuegen(self, von, nach, gewichtung, gerichtet):
        vonNummer = self.KnotenNummerGeben(von)
        nachNummer = self.KnotenNummerGeben(nach)
        if vonNummer != -1 and nachNummer != -1 and vonNummer != nachNummer:
            self.matrix[vonNummer][nachNummer] = gewichtung
            if not gerichtet:
                self.matrix[nachNummer][vonNummer] = gewichtung
            self.kanten.append(KantenSymbol(self.knoten[vonNummer].SymbolGeben(), self.knoten[nachNummer].SymbolGeben(), gerichtet, str(gewichtung), 3, "blau"))

    # Gibt die Adjazenzmatrix des Graphen in der Konsole aus
    def Ausgeben(self):
        # Kopfzeile
        print("      ", end='')
        for k in self.knoten:
            print((k.BezeichnerGeben() + "      ")[0:6], end='')
        print("")

        for i in range(len(self.knoten)):
            print((self.knoten[i].BezeichnerGeben() + "      ")[0:6], end='')
            for j in range(len(self.knoten)):
                if self.matrix[i][j] != -1:
                    print((str(self.matrix[i][j]) + "   ")[0:4], end='')
                else:
                    print("    ", end='')
            print("")

    # Gibt die Anzahl der Knoten des Graphen zurück
    def KnotenAnzahlGeben(self):
        return len(self.knoten)

    # Gibt die Gewichtung einer Kante zurück
    # Die Kante ist durch einen Anfangsknoten und einen Endknoten festgelegt
    def KanteGewichtGeben(self, von, nach):
        vonNummer = self.KnotenNummerGeben(von)
        nachNummer = self.KnotenNummerGeben(nach)
        if vonNummer != -1 and nachNummer!=-1:
            return self.matrix[vonNummer][nachNummer]
        else:
            return -1

    # Löscht die Kanten und Knoten des Graphen
    # Die Anzeige wird auch gelöscht
    def ZurueckSetzen(self):
        for k in self.kanten:
            k.Entfernen()
        for k in self.knoten:
            k.SymbolGeben().Entfernen()
        self.knoten.clear()
        self.kanten.clear()
        self.matrix.clear()
    
    # Führt die Breitensuche vom Startknoten zum Zielknoten durch
    def BreitensucheAusfuehren(self, fuellmenge):
        # Entsprechend der Teilaufgabe b ergänzen
        pass


# Beispiele für diverse Graphen
class Beispiele:

    # Legt das Graphenobjekt an.
    def __init__(self):
        self.g = GraphMatrix()
        self.l = Lesen()

    # Legt den Graphen für Kapitel 1.1 Aufgabe 10a an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenGraphA(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("GraphA.grdb", self.g):
            self.g.Ausgeben()

    # Legt den Graphen für Kapitel 1.1 Aufgabe 10b an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenGraphB(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("GraphB.grdb", self.g):
            self.g.Ausgeben()

    # Führt die Breitensuche vom Startknoten zum Zielknoten durch
    def BreitensucheAusfuehren(self, fuellmenge):
        if self.g is not None:
            if self.g.BreitensucheAusfuehren(fuellmenge):
                print("Die gewünschte Füllmenge wurde gefunden")
            else:
                print("Die gewünschte Füllmenge wurde nicht gefunden")
    
    # Führt die Breitensuche für jede mögliche Füllmenge (1 bis Kapazität des größten Gefäßes) durch.
    def AllesPruefen(self):
        # Gemäß Teilaufgabe c ergänzen
        pass
