from helferklassen import KnotenSymbol, KantenSymbol, Lesen



# Beschreibt einen Knoten
class Knoten:

    # Besetzt die Attribute und legt das Knotensymbol an.
    def __init__(self, bezeichner, x, y):
        self.bezeichner = bezeichner
        self.symbol = KnotenSymbol(x, y, 20, "weiss", bezeichner)

    # Meldet den Bezeichner des Knotens zurück.
    def BezeichnerGeben(self):
        return self.bezeichner

    # Meldet das Symbol des Knotens zurück.
    def SymbolGeben(self):
        return self.symbol

    # Setzt die Farbe der Darstellung
    def FarbeSetzen(self, f):
        self.symbol.FarbeSetzen(f)


class GraphMatrix:

    # Baut die Datenstruktur auf
    def __init__(self):
        self.knoten = []
        self.kanten = []
        self.matrix = []
        self.warteliste = []
        self.fertigeKnoten = []

    # Einfügen eines neuen Knoten in den Graphen
    def KnotenEinfuegen(self, bezeichner, x, y):
        self.knoten.append(Knoten(bezeichner, x, y))
        for zeile in self.matrix:
            zeile.append(-1)
        self.matrix.append([])
        for zeile in self.matrix:
            self.matrix[-1].append(-1)

    # Gibt den Knoten mit dem angegebenen Bezeichner zurück.
    def KnotenGeben(self, bezeichner):
        for k in self.knoten:
            if k.BezeichnerGeben() == bezeichner:
                return k
        return None

    # Gibt die interne Nummer des Knotens
    def KnotenNummerGeben(self, bezeichner):
        for i in range(len(self.knoten)):
            if self.knoten[i].BezeichnerGeben() == bezeichner:
                return i
        return -1

    # Gibt die Bezeichnung eines Knotens mit der internen Knotennummer
    def KnotenBezeichnerGeben(self, KnotenNummer):
        if knotenNummer < len(self.knoten) and knotenNummer >= 0:
            return self.knoten[knotenNummer].BezeichnerGeben()
        else:
            return ""

    # Einfügen einer Kante in den Graphen
    def KanteEinfuegen(self, von, nach, gewichtung):
        vonNummer = self.KnotenNummerGeben(von)
        nachNummer = self.KnotenNummerGeben(nach)
        if vonNummer != -1 and nachNummer != -1 and vonNummer != nachNummer:
            self.matrix[vonNummer][nachNummer] = gewichtung
            self.matrix[nachNummer][vonNummer] = gewichtung
            self.kanten.append(KantenSymbol(self.knoten[vonNummer].SymbolGeben(), self.knoten[nachNummer].SymbolGeben(), False, str(gewichtung), 3, "blau"))

    # Gibt die Adjazenzmatrix des Graphen in der Konsole aus
    def Ausgeben(self):
        # Kopfzeile
        print("    ", end='')
        for k in self.knoten:
            print((k.BezeichnerGeben() + "    ")[0:4], end='')
        print("")

        for i in range(len(self.knoten)):
            print((self.knoten[i].BezeichnerGeben() + "    ")[0:4], end='')
            for j in range(len(self.knoten)):
                if self.matrix[i][j] != -1:
                    print((str(self.matrix[i][j]) + "   ")[0:4], end='')
                else:
                    print("    ", end='')
            print("")

    # Gibt die Anzahl der Knoten des Graphen zurück
    def KnotenAnzahlGeben(self):
        return len(self.knoten)

    # Gibt die Gewichtung einer Kante zurück
    # Die Kante ist durch einen Anfangsknoten und einen Endknoten festgelegt
    def KanteGewichtGeben(self, von, nach):
        vonNummer = self.KnotenNummerGeben(von)
        nachNummer = self.KnotenNummerGeben(nach)
        if vonNummer != -1 and nachNummer!=-1:
            return self.matrix[vonNummer][nachNummer]
        else:
            return -1

    # Löscht die Kanten und Knoten des Graphen
    # Die Anzeige wird auch gelöscht
    def ZurueckSetzen(self):
        for k in self.kanten:
            k.Entfernen()
        for k in self.knoten:
            k.SymbolGeben().Entfernen()
        self.knoten.clear()
        self.kanten.clear()
        self.matrix.clear()
    
    # Führt die Breitensuche vom Startknoten zum Zielknoten durch
    def BreitensucheAusfuehren(self, startKnoten, zielKnoten):
        self.warteliste.clear()
        self.fertigeKnoten.clear()
        aktuellerKnoten = startKnoten
        while aktuellerKnoten != zielKnoten and aktuellerKnoten != -1:
            for nummer in range(len(self.knoten)):
                if self.matrix[aktuellerKnoten][nummer] > 0 and not nummer in self.fertigeKnoten and not nummer in self.warteliste:
                    self.warteliste.append(nummer)
            self.fertigeKnoten.append(aktuellerKnoten)
            if len(self.warteliste) > 0:
                aktuellerKnoten = self.warteliste.pop(0)
            else:
                aktuellerKnoten = -1
        if aktuellerKnoten != -1:
            print("Ziel erreicht")
            self.knoten[aktuellerKnoten].FarbeSetzen("gruen")
        else:
            print("Ziel nicht erreichbar")


# Beispiele für diverse Graphen
class Beispiele:

    # Legt das Graphenobjekt an.
    def __init__(self):
        self.g = GraphMatrix()
        self.l = Lesen()

    # Legt den Graphen für das Beispiel im Lehrtext an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenAbiturfahrt(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("Abiturfahrt.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen Graphen für den süddeutschen Teil des Autobahnnetztes an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenAutobahn(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("Autobahnen.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen Graphen für dei gegebenen Flugverbindungen an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenFlug(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("Fluglinien.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen Graphen für das deutsche ICE-Netzt an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenICE(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("ICENetz.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen Graphen für das münchner S-U-Bahn-Netzt an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenSUBahn(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("SUBahn.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen zusammenhängenden Graphen an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenBeispielgraph(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("Beispielgraph.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen nicht zusammenhängenden Graphen an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenUngeeignetesBeispiel(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("Beispielgraph2.grdb", self.g):
            self.g.Ausgeben()
