from helferklassen import KnotenSymbol, KantenSymbol, Lesen

# Beschreibt einen Knoten
class Knoten:

    # Besetzt die Attribute und legt das Knotensymbol an.
    def __init__(self, bezeichner, x, y):
        self.bezeichner = bezeichner
        self.symbol = KnotenSymbol(x, y, 20, "weiss", bezeichner)
        self.adjazenzliste = []

    # Meldet den Bezeichner des Knotens zurück.
    def BezeichnerGeben(self):
        return self.bezeichner

    # Meldet das Symbol des Knotens zurück.
    def SymbolGeben(self):
        return self.symbol

    # Setzt die Farbe der Darstellung
    def FarbeSetzen(self, f):
        self.symbol.FarbeSetzen(f)
    
    # Meldet die Adjazenzliste des Knotens zurück.
    def AdjazenzlisteGeben(self):
        return self.adjazenzliste


# Beschreibt eine Kante für die Graphendarstellung mit Adjazenzlisten
class Kante:
    
    # Attribute laut Beschreibung ergänzen
    def __init__(self, gewicht, ziel):
        self.gewicht = gewicht;
        self.ziel = ziel
    
    # Meldet das Gewicht der Kante zurück
    def GewichtGeben(self):
        return self.gewicht
    
    # Meldet den Zielknoten der Kante zurück
    def ZielGeben(self):
        return self.ziel


# Verwaltet einen ungerichteten, gewichteten Graphen mittels Adjazenzlisten
class GraphListe:

    # Baut die Datenstruktur auf
    def __init__(self):
        self.knoten = []
        self.kanten = []
        # Initialisierung der neuen Felder ergänzen

    # Einfügen eines neuen Knoten in den Graphen
    def KnotenEinfuegen(self, bezeichner, x, y):
        self.knoten.append(Knoten(bezeichner, x, y))

    # Gibt den Knoten mit dem angegebenen Bezeichner zurück.
    def KnotenGeben(self, bezeichner):
        for k in self.knoten:
            if k.BezeichnerGeben() == bezeichner:
                return k
        return None

    # Gibt die interne Nummer des Knotens
    def KnotenNummerGeben(self, bezeichner):
        for i in range(len(self.knoten)):
            if self.knoten[i].BezeichnerGeben() == bezeichner:
                return i
        return -1

    # Gibt die Bezeichnung eines Knotens mit der internen Knotennummer
    def KnotenBezeichnerGeben(self, KnotenNummer):
        if knotenNummer < len(self.knoten) and knotenNummer >= 0:
            return self.knoten[knotenNummer].BezeichnerGeben()
        else:
            return ""

    # Einfügen einer Kante in den Graphen
    def KanteEinfuegen(self, von, nach, gewichtung):
        vonNummer = self.KnotenNummerGeben(von)
        nachNummer = self.KnotenNummerGeben(nach)
        if vonNummer != -1 and nachNummer != -1 and vonNummer != nachNummer:
            self.knoten[vonNummer].AdjazenzlisteGeben().append(Kante(gewichtung, self.knoten[nachNummer]))
            self.knoten[nachNummer].AdjazenzlisteGeben().append(Kante(gewichtung, self.knoten[vonNummer]))
            self.kanten.append(KantenSymbol(self.knoten[vonNummer].SymbolGeben(), self.knoten[nachNummer].SymbolGeben(), False, str(gewichtung), 3, "blau"))

    # Gibt die Adjazenzmatrix des Graphen in der Konsole aus
    def Ausgeben(self):
        for k in self.knoten:
            print(k.BezeichnerGeben() + ": ", end="")
            for ka in k.AdjazenzlisteGeben():
                print("(" + str(ka.GewichtGeben()) + "-> " + ka.ZielGeben().BezeichnerGeben() + ") ", end="")
            print("")

    # Gibt die Anzahl der Knoten des Graphen zurück
    def KnotenAnzahlGeben(self):
        return len(self.knoten)

    # Gibt die Gewichtung einer Kante zurück
    # Die Kante ist durch einen Anfangsknoten und einen Endknoten festgelegt
    def KanteGewichtGeben(self, von, nach):
        vonNummer = self.KnotenNummerGeben(von)
        nachKnoten = self.KnotenGeben(nach)
        if vonNummer != -1 and nachKnoten is not None:
            for k in self.knoten[vonNummer].AdjazenzlisteGeben():
                if k.ZielGeben() == nachKnoten:
                    return k.GewichtGeben()
            return -1
        else:
            return -1

    # Löscht die Kanten und Knoten des Graphen
    # Die Anzeige wird auch gelöscht
    def ZurueckSetzen(self):
        for k in self.kanten:
            k.Entfernen()
        for k in self.knoten:
            k.SymbolGeben().Entfernen()
        self.knoten.clear()
        self.kanten.clear()
    
    # Führt die Breitensuche vom Startknoten zum Zielknoten durch
    def BreitensucheAusfuehren(self, startKnoten, zielKnoten):
        # Algorithmus ergänzen
        pass


# Beispiele für diverse Graphen
class Beispiele:

    # Legt das Graphenobjekt an.
    def __init__(self):
        self.g = GraphListe()
        self.l = Lesen()

    # Legt den Graphen für das Beispiel im Lehrtext an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenAbiturfahrt(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("Abiturfahrt.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen Graphen für den süddeutschen Teil des Autobahnnetztes an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenAutobahn(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("Autobahnen.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen Graphen für dei gegebenen Flugverbindungen an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenFlug(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("Fluglinien.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen Graphen für das deutsche ICE-Netzt an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenICE(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("ICENetz.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen Graphen für das münchner S-U-Bahn-Netzt an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenSUBahn(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("SUBahn.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen zusammenhängenden Graphen an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenBeispielgraph(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("Beispielgraph.grdb", self.g):
            self.g.Ausgeben()

    # Legt einen nicht zusammenhängenden Graphen an.
    # Am Ende Ausgabe der Adjazenzmatrix zur Kontrolle
    def AusfuehrenUngeeignetesBeispiel(self):
        self.g.ZurueckSetzen()
        if self.l.LesenDatenbank("Beispielgraph2.grdb", self.g):
            self.g.Ausgeben()
    
    # Führt die Breitensuche vom Startknoten zum Zielknoten durch
    def BreitensucheAusfuehren (self, nameStartKnoten, nameZielKnoten):
        if self.g is not None:
            self.g.BreitensucheAusfuehren(self.g.KnotenGeben(nameStartKnoten), self.g.KnotenGeben(nameZielKnoten))
