from graphics_and_games_klassen import Kreis, Text, Rechteck, Dreieck, Zeichenfenster
import math, sqlite3


# Verwaltet ein Knotensymbol in einem Graph
class KnotenSymbol():

    # Legt das Symbol an und besetzt die Attribute.
    def __init__(self, x, y, radius, farbe, bezeichner):
        self.x = x
        self.y = y
        self.r = radius
        self.farbe = farbe
        self.bezeichner = bezeichner
        self.aussen = Kreis()
        self.aussen.FarbeSetzen("schwarz")
        self.innen = Kreis()
        self.text = Text()
        self.text.TextGroesseSetzen(18)
        self.DarstellungAktualisieren()

    # Aktualisiert das Symbol
    def DarstellungAktualisieren(self):
        self.aussen.PositionSetzen(self.x, self.y)
        self.aussen.RadiusSetzen(self.r)
        self.innen.PositionSetzen(self.x, self.y)
        self.innen.RadiusSetzen(self.r - 2)
        self.innen.FarbeSetzen(self.farbe)
        self.text.PositionSetzen(int(self.x - len(self.bezeichner) / 2 * 10 - len(self.bezeichner) % 2 * 5), self.y - 6)
        self.text.TextSetzen(self.bezeichner)

    # Setzt die Farbe der Darstellung
    def FarbeSetzen(self, f):
        self.farbe = f
        self.DarstellungAktualisieren()

    # Setzt die Position der Darstellung
    def PositionSetzen(self, x, y):
        self.x = x
        self.y = y
        self.DarstellungAktualisieren()

    # Setzt die Farbe der Darstellung
    def BezeichnerSetzen(self, bezeichner):
        self.bezeichner = bezeichner
        self.DarstellungAktualisieren()

    # Meldet die x-Koordinate des Symbols zurück.
    def XGeben(self):
        return self.x

    # Meldet die y-Koordinate des Symbols zurück.
    def YGeben(self):
        return self.y

    # Meldet den Bezeichner des Symbols zurück.
    def BezeichnerGeben(self):
        return self.bezeichner

    # Entfernt das Knotensymbol aus der Anzeige
    def Entfernen(self):
        self.aussen.Entfernen()
        self.innen.Entfernen()
        self.text.Entfernen()

# Verwaltet ein Kantensymbol in einem Graph
class KantenSymbol():

    # Legt das Symbol an und besetzt die Attribute.
    def __init__(self, start, ende, gerichtet, gewicht, breite, farbe):
        self.von = start
        self.nach = ende
        self.farbe = farbe
        self.breite = breite
        self.gerichtet = gerichtet
        self.gewicht = gewicht
        self.aussen = Rechteck()
        self.innen = Rechteck()
        self.pfeil = Dreieck()
        self.pfeil.FarbeSetzen("schwarz")
        self.text = Text()
        self.text.SichtbarkeitSetzen(gewicht != "")
        self.text.TextSetzen(gewicht)
        self.DarstellungAktualisieren()

    # Aktualisiert das Symbol
    def DarstellungAktualisieren(self):
        x1 = self.von.XGeben()
        y1 = self.von.YGeben()
        x2 = self.nach.XGeben()
        y2 = self.nach.YGeben()
        laenge = int(math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)))
        xm = int((x1 + x2) / 2.0)
        ym = int((y1 + y2) / 2.0)
        x = int(xm - laenge / 2)
        y = int(ym - self.breite / 2)
        self.aussen.PositionSetzen(x, y)
        self.aussen.GroesseSetzen(laenge, self.breite)
        if (self.breite < 4):
            groesse = self.breite * 4
            self.innen.SichtbarkeitSetzen(False)
            self.aussen.FarbeSetzen(self.farbe)
            self.pfeil.SichtbarkeitSetzen(self.gerichtet)
            self.pfeil.PositionSetzen(xm, int(ym - groesse / 2))
            self.pfeil.GroesseSetzen(groesse, groesse)
            self.pfeil.WinkelSetzen(270)
        else:
            delta = 2 if self.breite <= 8 else 4
            self.innen.SichtbarkeitSetzen(True)
            self.innen.PositionSetzen(x, int(y + delta / 2))
            self.innen.GroesseSetzen(laenge, self.breite - delta)
            self.innen.FarbeSetzen(self.farbe)
            self.aussen.FarbeSetzen("schwarz")
            self.pfeil.SichtbarkeitSetzen(self.gerichtet)
            self.pfeil.PositionSetzen(xm, int(ym - self.breite / 2))
            self.pfeil.GroesseSetzen(self.breite - delta, self.breite)
            self.pfeil.WinkelSetzen(270)

        winkel = 0.0
        if x1 == x2:
            if y2 > y1:
                winkel = -90
            else:
                winkel = 90
        elif x1 < x2:
            winkel = -math.atan((y2 - y1) / (x2 - x1)) / math.pi * 180.0
        else:
            winkel = 180 - math.atan((y2 - y1) / (x2 - x1)) / math.pi * 180.0
        self.pfeil.WinkelSetzen(int(winkel + 270))
        self.innen.WinkelSetzen(int(winkel))
        self.aussen.WinkelSetzen(int(winkel))
        self.pfeil.GanzNachHintenBringen()
        self.innen.GanzNachHintenBringen()
        self.aussen.GanzNachHintenBringen()
        if ((-90 <= winkel) and (winkel <= 0)):
            w = (winkel + 90) / 180.0 * math.pi
            delta = self.breite
            self.text.PositionSetzen(xm + int(delta * math.cos(w)), ym - int(delta * math.sin(w)))
        elif ((0 < winkel) and (winkel <= 90)):
            w = (winkel + 90) / 180.0 * math.pi
            delta = self.breite
            self.text.PositionSetzen(xm - int(delta * math.cos(w)), ym + int(delta * math.sin(w)) + 7)
        elif ((90 < winkel) and (winkel <= 180)):
            w = (winkel + 90) / 180.0 * math.pi
            delta = self.breite
            self.text.PositionSetzen(xm - int(delta * math.cos(w)), ym + int(delta * math.sin(w)))
        else:
            w = (winkel + 90) / 180.0 * math.pi
            delta = self.breite
            self.text.PositionSetzen(xm + int(delta * math.cos(w)), ym - int(delta * math.sin(w)) + 10)
    
    # Entfernt das Kantensymbol aus der Anzeige
    def Entfernen(self):
        self.aussen.Entfernen()
        self.innen.Entfernen()
        self.pfeil.Entfernen()
        self.text.Entfernen()

    # Setzt die Farbe der Darstellung
    def FarbeSetzen(self, f):
        self.farbe = f
        self.DarstellungAktualisieren()

    # Meldet den Startknoten
    def StartsymbolGeben(self):
        return self.von

    # Meldet den Zielknoten
    def ZielsymbolGeben(self):
        return self.nach


# Beschreibt einen Knoten
class Knoten:

    # Besetzt die Attribute und legt das Knotensymbol an.
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.symbol = KnotenSymbol(int(x * GartenElement.seitenlaenge + GartenElement.seitenlaenge / 2), int(Zeichenfenster.MalflaechenHoeheGeben() / 2 + y * GartenElement.seitenlaenge + GartenElement.seitenlaenge / 2), int((GartenElement.seitenlaenge - 4) / 2), "weiss", "")

    # Meldet das Symbol des Knotens zurück.
    def SymbolGeben(self):
        return self.symbol

    # Setzt die Farbe der Darstellung
    def FarbeSetzen(self, f):
        self.symbol.FarbeSetzen(f)
    
    # Meldet die x-Koordinate des Knotens zurück.
    def XGeben(self):
        return self.x
    
    # Meldet die y-Koordinate des Knotens zurück.
    def YGeben(self):
        return self.y


# Oberklasse aller Bausteine des Gartens
class GartenElement:
    seitenlaenge = 20

    # Legt das Darstellungsrechteck an.
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.darstellung = Rechteck()
        self.darstellung.PositionSetzen(x * GartenElement.seitenlaenge, y * GartenElement.seitenlaenge);
        self.darstellung.GroesseSetzen(GartenElement.seitenlaenge, GartenElement.seitenlaenge)
    
    # Gibt die x-Position des Elements zurück.
    def XPositionGeben(self):
        return self.x
    
    # Gibt die y-Position des Elements zurück.
    def YPositionGeben(self):
        return self.y


# Ein Rasenstück des Gartens
class Rasen(GartenElement):
    
    # Legt das Darstellungsrechteck in der Rasenfarbe an.
    def __init__(self, x, y):
        super().__init__(x, y)
        self.darstellung.FarbeSetzen("gruen")


# Ein Hinderniss des Gartens
class Hindernis(GartenElement):
    
    # Legt das Darstellungsrechteck in der Hindernisfarbe an.
    def __init__(self, x, y):
        super().__init__(x, y)
        self.darstellung.FarbeSetzen("braun")


# Verwaltet den Garten
class Garten:

    # Legt den Garten an.
    def __init__(self, dateiName, graph):
        self.elemente = []
        if self.Lesen(dateiName):
            self.GraphErzeugen(graph)

    # Liest die Beschreibungs des Gartens aus einer Datei ein
    def Lesen(self, dateiName):
        try:
            with open(dateiName, 'r') as datei:
                zeilen = [zeile.rstrip().lower() for zeile in datei]
                y = 0
                for zeile in zeilen:
                    self.elemente.append([])
                    x = 0
                    for ch in zeile:
                        if ch == 'h':
                            e = Hindernis(x, y)
                        elif ch == 'r':
                            e = Rasen(x, y)
                        else:
                            e = GartenElement(x, y)
                            e.darstellung.FarbeSetzen("rot")
                        self.elemente[y].append(e)
                        x += 1
                    y += 1
            return True
        except Exception as e:
            t = Text()
            t.PositionSetzen(10, Zeichenfenster.MalflaechenHoeheGeben() - 30)
            t.TextSetzen("Fehler beim Lesen der Datei '" + dateiName + "': " + str(e))
            return False
    
    # Erzeugt die Graphendarstellung
    def GraphErzeugen(self, graph):
        for y in range(len(self.elemente)):
            for x in range(len(self.elemente[y])):
                if isinstance(self.elemente[y][x], Rasen):
                    graph.KnotenEinfuegen(x, y)
                    if y > 0 and isinstance(self.elemente[y - 1][x], Rasen):
                        graph.KanteEinfuegen(x, y - 1, x, y)
                    if x > 0 and isinstance(self.elemente[y][x - 1], Rasen):
                        graph.KanteEinfuegen(x - 1, y, x, y)
